import boto3
from typing import Any, Union

from navalmartin_mir_aws_utils.exceptions import InvalidAWSClientException
from navalmartin_mir_aws_utils.aws_credentials import (
    AWSCredentials_S3,
    AWSCredentials_SQS,
    AWSCredentials_CognitoIDP,
    AWSCredentials_SecretsManager,
    AWSCredentials_SES,
    AWSCredentials_SFN,
)

VALID_AWS_CLIENTS = ["s3", "sqs", "cognito-idp", "secretsmanager", "ses", "sfn"]


def get_aws_client_factory(
    credentials: Union[
        AWSCredentials_S3,
        AWSCredentials_SQS,
        AWSCredentials_CognitoIDP,
        AWSCredentials_SecretsManager,
        AWSCredentials_SES,
        AWSCredentials_SFN,
    ]
) -> Any:
    if credentials.aws_client_name not in VALID_AWS_CLIENTS:
        raise InvalidAWSClientException(
            client_name=credentials.aws_client_name, allowed_vals=VALID_AWS_CLIENTS
        )

    if credentials.aws_client_name == "s3":
        return get_aws_s3_client(credentials=credentials)
    elif credentials.aws_client_name == "sqs":
        return get_aws_sqs_client(credentials=credentials)
    elif credentials.aws_client_name == "cognito-idp":
        return get_aws_cognito_idp_client(credentials=credentials)
    elif credentials.aws_client_name == "secretsmanager":
        return get_aws_secrets_manager_client(credentials=credentials)
    elif credentials.aws_client_name == "ses":
        return get_aws_ses_client(credentials=credentials)
    elif credentials.aws_client_name == "sfn":
        return get_aws_sfn_client(credentials=credentials)

    return None


def get_aws_s3_client(credentials: AWSCredentials_S3):
    """Get the AWS S3 client

    Parameters
    ----------
    credentials: Credentials for accessing the S3 service
    Returns
    -------
    """

    if credentials.aws_region == "" or credentials.aws_region is None:
        raise ValueError(
            "Invalid region name in credentials. "
            "'credentials.aws_region' cannot be '' or None"
        )

    if (
        credentials.aws_secret_access_key is not None
        and credentials.aws_access_key is not None
    ):
        return boto3.client(
            credentials.aws_client_name,
            aws_access_key_id=credentials.aws_access_key,
            aws_secret_access_key=credentials.aws_secret_access_key,
            region_name=credentials.aws_region,
        )

    return boto3.client(credentials.aws_client_name, region_name=credentials.aws_region)


def get_aws_sqs_client(credentials: AWSCredentials_SQS):
    """Get the AWS SQS client

    Parameters
    ----------
    credentials: Credentials for accessing the SQS service

    Returns
    -------

    """

    if credentials.aws_region == "" or credentials.aws_region is None:
        raise ValueError(
            "Invalid region name in credentials. "
            "'credentials.aws_region' cannot be '' or None"
        )

    if (
        credentials.aws_secret_access_key is not None
        and credentials.aws_access_key is not None
    ):
        return boto3.client(
            credentials.aws_client_name,
            aws_access_key_id=credentials.aws_access_key,
            aws_secret_access_key=credentials.aws_secret_access_key,
            region_name=credentials.aws_region,
        )

    return boto3.client(credentials.aws_client_name, region_name=credentials.aws_region)


def get_aws_cognito_idp_client(credentials: AWSCredentials_CognitoIDP):
    """Get the Cognito IDP client for the given credentials

    Parameters
    ----------
    credentials: Credentials for accessing the Cognito IDP service

    """

    if credentials.aws_region == "" or credentials.aws_region is None:
        raise ValueError(
            "Invalid region name in credentials. "
            "'credentials.aws_region' cannot be '' or None"
        )

    if (
        credentials.aws_secret_access_key is not None
        and credentials.aws_access_key is not None
    ):
        return boto3.client(
            credentials.aws_client_name,
            aws_access_key_id=credentials.aws_access_key,
            aws_secret_access_key=credentials.aws_secret_access_key,
            region_name=credentials.aws_region,
        )

    return boto3.client(credentials.aws_client_name, credentials.aws_region)


def get_aws_secrets_manager_client(credentials: AWSCredentials_SecretsManager) -> Any:
    """

    Parameters
    ----------
    credentials

    Returns
    -------

    """

    if credentials.aws_region == "" or credentials.aws_region is None:
        raise ValueError(
            "Invalid region name in credentials. "
            "'credentials.aws_region' cannot be '' or None"
        )

    if (
        credentials.aws_secret_access_key is not None
        and credentials.aws_access_key is not None
    ):
        return boto3.client(
            credentials.aws_client_name,
            aws_access_key_id=credentials.aws_access_key,
            aws_secret_access_key=credentials.aws_secret_access_key,
            region_name=credentials.aws_region,
        )

    return boto3.client(credentials.aws_client_name, region_name=credentials.aws_region)


def get_aws_ses_client(credentials: AWSCredentials_SES) -> Any:
    if credentials.aws_region == "" or credentials.aws_region is None:
        raise ValueError(
            "Invalid region name in credentials. "
            "'credentials.aws_region' cannot be '' or None"
        )

    if (
        credentials.aws_secret_access_key is not None
        and credentials.aws_access_key is not None
    ):
        return boto3.client(
            credentials.aws_client_name,
            aws_access_key_id=credentials.aws_access_key,
            aws_secret_access_key=credentials.aws_secret_access_key,
            region_name=credentials.aws_region,
        )

    return boto3.client(credentials.aws_client_name, region_name=credentials.aws_region)


def get_aws_sfn_client(credentials: AWSCredentials_SFN) -> Any:
    if credentials.aws_region == "" or credentials.aws_region is None:
        raise ValueError(
            "Invalid region name in credentials. "
            "'credentials.aws_region' cannot be '' or None"
        )

    if (
        credentials.aws_secret_access_key is not None
        and credentials.aws_access_key is not None
    ):
        return boto3.client(
            credentials.aws_client_name,
            aws_access_key_id=credentials.aws_access_key,
            aws_secret_access_key=credentials.aws_secret_access_key,
            region_name=credentials.aws_region,
        )

    return boto3.client(credentials.aws_client_name, region_name=credentials.aws_region)



from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_S3
from navalmartin_mir_aws_utils.s3_utils import (
    delete_s3_all_objs_with_key,
    delete_s3_object_with_key,
)

AWS_REGION = "YOUR_AWS_REGION"
AWS_S3_BUCKET_NAME = "YOUR_AWS_S3_BUCKET_NAME"
AWS_ACCESS_KEY = "YOUR_AWS_ACCESS_KEY"
AWS_SECRET_ACCESS_KEY = "YOUR_AWS_SECRET_ACCESS_KEY"

if __name__ == "__main__":
    # credentials so that we can access S3
    aws_s3_credentials = AWSCredentials_S3(
        aws_s3_bucket_name=AWS_S3_BUCKET_NAME,
        aws_region=AWS_REGION,
        aws_access_key=AWS_ACCESS_KEY,
        aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
    )

    print("---------Deleting one object")

    # delete an object from S3
    key = "63b5bc8cc5f1cdcad7ef54e7/2/2/img_1.jpeg"
    delete_response = delete_s3_object_with_key(
        key=key, s3_client=None, aws_creds=aws_s3_credentials
    )

    print(delete_response)
    print("---------Deleting multiple objects")
    keys = [
        {"Key": "63b5bc8cc5f1cdcad7ef54e7/2/2/img_2.jpeg"},
        {"Key": "63b5bc8cc5f1cdcad7ef54e7/2/2/img_3.jpeg"},
    ]
    delete_response = delete_s3_all_objs_with_key(
        keys=keys, s3_client=None, aws_creds=aws_s3_credentials
    )
    print(delete_response)



from typing import Any
from navalmartin_mir_aws_utils import AWSCredentials_SES
from navalmartin_mir_aws_utils import get_aws_client_factory
from navalmartin_mir_aws_utils.utils import Email


def send_simple_email(
    email: Email, credentials: AWSCredentials_SES, ses_client: Any = None
):
    if ses_client is None:
        ses_client = get_aws_client_factory(credentials=credentials)

    # format the email and send it
    response = ses_client.send_email(
        Source=email.source,
        Destination={
            "ToAddresses": [
                email.destination,
            ],
            "CcAddresses": email.ccs,
        },
        ReplyToAddresses=[
            email.reply_to,
        ],
        Message={
            "Subject": {"Data": email.subject.data},
            "Body": {"Text": {"Data": email.body.data}},
        },
    )
    return response



from .tasks_lambdas_utils import get_error_return, get_success_return, get_secrets
from .tasks_lambdas_utils import validate_sqs_event_record
from .aws_tasks_decorator import (
    aws_lambda_task_decorator,
    async_aws_lambda_task_decorator,
)



"""
Various utilities for working with AWS in Python. The official
PyPi package can be found <a href="https://pypi.org/project/navalmartin-mir-aws-utils/">here</a>.
This package is basically wrappers on top of boto3 that allows for easier development with AWS.
"""
__version__ = "0.0.26"
from .boto3_client import get_aws_client_factory
from .aws_credentials import (
    AWSCredentials_S3,
    AWSCredentials_SQS,
    AWSCredentials_CognitoIDP,
    AWSCredentials_SecretsManager,
    AWSCredentials_SES,
    AWSCredentials_SFN,
)
from .file_s3_batch import FilePathBatch



from pydantic import EmailStr
from navalmartin_mir_aws_utils import AWSCredentials_SES
from navalmartin_mir_aws_utils.tasks import send_simple_email
from navalmartin_mir_aws_utils.utils import Email, EmailSubject, EmailBody


if __name__ == "__main__":
    credentials = AWSCredentials_SES(aws_region="eu-west-2")
    email = Email(
        source=EmailStr("some_email@provider.com"),
        destination=EmailStr("some_email@provider.com"),
        subject=EmailSubject(data="First notification from mir"),
        body=EmailBody(
            data="This your first AWS SES notification from mir. Expect more"
        ),
        reply_to=EmailStr("alex@navalmartin.com"),
    )

    print(email.dict())
    response = send_simple_email(email=email, credentials=credentials)
    print(response)
